/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.text.ri.combined;

import java.util.Locale;
import javax.swing.event.EventListenerList;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.MissingDependencyException;
import org.jdtaus.core.container.MissingPropertyException;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.text.MessageEvent;
import org.jdtaus.core.text.MessageListener;
import org.jdtaus.core.text.spi.ApplicationLogger;
import org.jdtaus.core.logging.spi.Logger;

/**
 * jDTAUS Core SPI {@code ApplicationLogger} implementation.
 * <p>The combined implementation will log messages with the logger configured
 * as a dependency to this class and then notifies all registered listeners
 * about these messages.</p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: CombinedApplicationLogger.java 2522 2007-04-12 09:28:25Z schulte2005 $
 */
public class CombinedApplicationLogger
    implements ApplicationLogger, ContainerInitializer
{

    //--Constants---------------------------------------------------------------

    /** A technical logger to log application messages with. */
    public interface TechLogger
    {

        /**
         * Getter for property {@code levelName}.
         *
         * @return logical name of the level this logger uses.
         */
        String getLevelName();

        /**
         * Logs {@code message} using {@code logger} with the level determined
         * by property {@code levelName}.
         *
         * @param logger the logger to use.
         * @param message the message to log.
         *
         * @throws NullPointerException if either {@code logger} or
         * {@code message} is {@code null}.
         */
        void log(Logger logger, String message);

    }

    /** Technical trace message logger. */
    private static final TechLogger TRACE = new TechLogger()
    {

        public String getLevelName()
        {
            return "trace";
        }

        public void log(final Logger logger, final String message)
        {
            if(logger == null)
            {
                throw new NullPointerException("logger");
            }
            if(message == null)
            {
                throw new NullPointerException("message");
            }

            logger.trace(message);
        }
    };

    /** Technical debug message logger. */
    private static final TechLogger DEBUG = new TechLogger()
    {

        public String getLevelName()
        {
            return "debug";
        }


        public void log(final Logger logger, final String message)
        {
            if(logger == null)
            {
                throw new NullPointerException("logger");
            }
            if(message == null)
            {
                throw new NullPointerException("message");
            }

            logger.debug(message);
        }
    };

    /** Technical info message logger. */
    private static final TechLogger INFO = new TechLogger()
    {

        public String getLevelName()
        {
            return "info";
        }


        public void log(final Logger logger, final String message)
        {
            if(logger == null)
            {
                throw new NullPointerException("logger");
            }
            if(message == null)
            {
                throw new NullPointerException("message");
            }

            logger.info(message);
        }
    };

    /** Technical warning message logger. */
    private static final TechLogger WARN = new TechLogger()
    {

        public String getLevelName()
        {
            return "warn";
        }


        public void log(final Logger logger, final String message)
        {
            if(logger == null)
            {
                throw new NullPointerException("logger");
            }
            if(message == null)
            {
                throw new NullPointerException("message");
            }

            logger.warn(message);
        }
    };

    /** Technical error message logger. */
    private static final TechLogger ERROR = new TechLogger()
    {

        public String getLevelName()
        {
            return "error";
        }


        public void log(final Logger logger, final String message)
        {
            if(logger == null)
            {
                throw new NullPointerException("logger");
            }
            if(message == null)
            {
                throw new NullPointerException("message");
            }

            logger.error(message);
        }
    };

    /** Technical fatal message logger. */
    private static final TechLogger FATAL = new TechLogger()
    {

        public String getLevelName()
        {
            return "fatal";
        }

        public void log(final Logger logger, final String message)
        {
            if(logger == null)
            {
                throw new NullPointerException("logger");
            }
            if(message == null)
            {
                throw new NullPointerException("message");
            }

            logger.fatal(message);
        }
    };

    /** Possible values for property {@code logLevel}. */
    private static final TechLogger[] TECH_LOGGERS = {
        TRACE, DEBUG, INFO, WARN, ERROR, FATAL
    };

    //---------------------------------------------------------------Constants--
    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(CombinedApplicationLogger.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Protected <code>CombinedApplicationLogger</code> implementation constructor.
    * @param meta Implementation meta-data.
    */
    protected CombinedApplicationLogger(final Implementation meta)
    {
        super();
        Property p;

        p = meta.getProperties().getProperty("logLanguage");
        this._logLanguage = (java.lang.String) p.getValue();


        p = meta.getProperties().getProperty("logLevel");
        this._logLevel = (java.lang.String) p.getValue();

    }
    /** Protected <code>CombinedApplicationLogger</code> dependency constructor.
    * @param meta Dependency meta-data.
    */
    protected CombinedApplicationLogger(final Dependency meta)
    {
        super();
        Property p;

        p = meta.getProperties().getProperty("logLanguage");
        this._logLanguage = (java.lang.String) p.getValue();


        p = meta.getProperties().getProperty("logLevel");
        this._logLevel = (java.lang.String) p.getValue();

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /**
     * Initializes the instance.
     *
     * @see #assertValidProperties()
     */
    public void initialize()
    {
        this.assertValidProperties();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /**
     * Property {@code logLanguage}.
     * @serial
     */
    private java.lang.String _logLanguage;

    /** <code>logLanguage</code> property getter. */
    protected java.lang.String getLogLanguage()
    {
        return this._logLanguage;
    }

    /**
     * Property {@code logLevel}.
     * @serial
     */
    private java.lang.String _logLevel;

    /** <code>logLevel</code> property getter. */
    protected java.lang.String getLogLevel()
    {
        return this._logLevel;
    }


    //--------------------------------------------------------------Properties--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>Logger</code> implementation. */
    private transient Logger _dependency0;

    /** <code>Logger</code> implementation getter. */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(CombinedApplicationLogger.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(CombinedApplicationLogger.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--MessageEventSource------------------------------------------------------

    /** Holds {@code MessageListener}s. */
    private EventListenerList listeners = new EventListenerList();

    public final void addMessageListener(
        final MessageListener messageListener)
    {
        if(messageListener == null)
        {
            throw new NullPointerException("messageListener");
        }

        this.listeners.add(MessageListener.class, messageListener);
    }

    public final void removeMessageListener(
        MessageListener messageListener)
    {
        if(messageListener == null)
        {
            throw new NullPointerException("messageListener");
        }

        this.listeners.remove(MessageListener.class, messageListener);
    }

    public final MessageListener[] getMessageListeners()
    {
        return (MessageListener[]) this.listeners.
            getListeners(MessageListener.class);

    }

    /**
     * Notifies all registered {@code MessageListener}s.
     *
     * @param e the event to provide to all listeners.
     *
     * @throws NullPointerException if {@code e} is {@code null}.
     */
    protected final void fireOnMessage(final MessageEvent e)
    {
        if(e == null)
        {
            throw new NullPointerException("e");
        }

        final MessageListener[] listeners = this.getMessageListeners();
        for(int i = listeners.length - 1; i >= 0; i--)
        {
            listeners[i].onMessage(e);
        }
    }

    //------------------------------------------------------MessageEventSource--
    //--ApplicationLogger-------------------------------------------------------

    public void log(final MessageEvent messageEvent)
    {
        if(messageEvent == null)
        {
            throw new NullPointerException("messageEvent");
        }

        this.getTechLogger().log(this.getLogger(),
            messageEvent.getMessage().getText(this.getLocale()));

        this.fireOnMessage(messageEvent);
    }

    //-------------------------------------------------------ApplicationLogger--
    //--CombinedApplicationLogger-----------------------------------------------

    /** Creates a new {@code CombinedApplicationLogger} instance. */
    public CombinedApplicationLogger()
    {
        this(CombinedApplicationLogger.META);
        this.initialize();
    }

    /**
     * Gets a {@code TechLogger} for the log level defined by property
     * {@code logLevel}.
     *
     * @return a {@code TechLogger} suitable for the log level defined by the
     * value of property {@code logLevel} or {@code null} if property
     * {@code logLevel} holds an unsupported value.
     */
    protected TechLogger getTechLogger()
    {
        TechLogger ret = null;

        for(int i = CombinedApplicationLogger.
            TECH_LOGGERS.length - 1; i >= 0; i--)
        {

            if(CombinedApplicationLogger.TECH_LOGGERS[i].getLevelName().
                equalsIgnoreCase(this.getLogLevel()))
            {

                ret = CombinedApplicationLogger.TECH_LOGGERS[i];
                break;
            }
        }

        return ret;
    }

    /**
     * Gets the locale to use for logging messages with the {@code TechLogger}.
     *
     * @return the locale to use for logging messages using the
     * {@code TechLogger}.
     */
    protected Locale getLocale()
    {
        return new Locale(this.getLogLanguage());
    }

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    protected void assertValidProperties()
    {
        if(this.getLogLevel() == null || this.getTechLogger() == null)
        {
            throw new PropertyException("logLevel", this.getLogLevel());
        }
        if(this.getLogLanguage() == null ||
            this.getLogLanguage().length() <= 0)
        {

            throw new PropertyException("logLanguage", this.getLogLanguage());
        }
    }

    //-----------------------------------------------CombinedApplicationLogger--

}
